<?php
/**
 * @package     NextGenEditor
 *
 * @copyright   Copyright (C) Teatis. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      TEATIS - www.nextgeneditor.com
 */

defined('_JEXEC') or die;

require_once(NGEPATH_CONTENTS . '/source/contentsource.php');
require_once NGEPATH_HELPERS . '/ngedata.php'; //For mysql queries

class NgeDatabaseQueryContent extends NgeContentSource
{
    
	 /**
	 * Get data and format it to be rendered in a "thumbnails" like widget
	 * @param type $params
	 * @return \NgeContentStructureTabs
	 */
    static function getTabsData($params){
		
	require_once (NGEPATH_HELPERS . '/data/database/databasedriver.php');
	require_once (NGEPATH_CONTENTS . '/structure/tabs.php');

	$items = array();
	$query = $params->content->database_query->database->query;
	    
	    if ($query != ''){
		if (isset($params->content->content->count) && $params->content->content->count != '0')
		    $query .= ' LIMIT 0,' . $params->content->content->count;
		
		$data = NGEDATA::loadAssocList($query);

		switch ($params->content->database_query->layout->mode) {
		    case 'template' :
			    require_once (NGEPATH_HELPERS . '/renderer/mustache/mustacherenderer.php');
			    foreach ($data as $contentRow) {
				    $item = new NgeContentStructureTabs();
				    if ($params->content->database_query->layout->template->label != '')
					$item->tab->label = MustacheRenderer::renderString($contentRow, $params->content->database_query->layout->template->label);
				    if ($params->content->database_query->layout->template->body != '')
					$item->body = MustacheRenderer::renderString($contentRow, $params->content->database_query->layout->template->body);
				    /*else
				    if ($data->content->database_query->layout->template->source == 'file')
					    $item->body = SmartyRenderer::render($contentRow, $data->content->database_query->layout->template->file);*/
				    $items[] = $item;
			    }
			    break;
		    default : //fields
			    foreach ($data as $row) {
				    $item = new NgeContentStructureTabs();
				    $item->body = '<table class="table">';
				    $firstField = true;
				    foreach ($row as $key => $value) {
					    if ($firstField == true) {
						    $item->tab->label = $value;;
						    $firstField = false;
					    }
					    $item->body .= '<tr><td>' . $key . '</td><td>' . $value . '</td></tr>';
				    }
				    $item->body .= '</table>';
				    $items[] = $item;
			    }
			    break;
		}
			    
	    }
	    return $items;

    }

    /**
    * Get data and format it to be rendered in a "thumbnails" like widget
    * @param type $params
    * @return \NgeContentStructureThumbnails
    */
    static function getThumbnailsData($params){
		
	require_once (NGEPATH_HELPERS . '/data/database/databasedriver.php');
	require_once (NGEPATH_CONTENTS . '/structure/thumbnails.php');

	$items = array();
	$query = $params->content->database_query->database->query;
	    
	    if ($query != ''){
		if (isset($params->content->content->count) && $params->content->content->count != '0')
		    $query .= ' LIMIT 0,' . $params->content->content->count;
		
		$data = NGEDATA::loadAssocList($query);

		switch ($params->content->database_query->layout->mode) {
		    case 'template' :
			    require_once (NGEPATH_HELPERS . '/renderer/mustache/mustacherenderer.php');
			    foreach ($data as $contentRow) {
				    $item = new NgeContentStructureThumbnails();
				    if ($params->content->database_query->layout->template->label != '')
					$item->header->label = MustacheRenderer::renderString($contentRow, $params->content->database_query->layout->template->label);
				    if ($params->content->database_query->layout->template->body != '')
					$item->body = MustacheRenderer::renderString($contentRow, $params->content->database_query->layout->template->body);
				    if ($params->content->database_query->layout->template->caption != '')
					$item->caption = MustacheRenderer::renderString($contentRow, $params->content->database_query->layout->template->caption);
				    /*else
				    if ($data->content->database_query->layout->template->source == 'file')
					    $item->body = SmartyRenderer::render($contentRow, $data->content->database_query->layout->template->file);*/
				    $items[] = $item;
			    }
			    break;
		    default : //fields
			    foreach ($data as $row) {
				    $item = new NgeContentStructureThumbnails();
				    $item->body = '<table class="table">';
				    $firstField = true;
				    foreach ($row as $key => $value) {
					    if ($firstField == true) {
						    $labels[] = $value;
						    $firstField = false;
					    }
					    $item->body .= '<tr><td>' . $key . '</td><td>' . $value . '</td></tr>';
				    }
				    $item->body .= '</table>';
				    $items[] = $item;
			    }
			    break;
		}
	    }
	    return $items;

    }
    
    
	static function getLabel($data, $params){

	    switch ($params->layout->mode) {
		case 'fields' :
		    //Default label: first field
		    return current($data);
		    break;
		case 'template' :
		    require_once (NGEPATH_HELPERS . '/renderer/smarty/smartyrenderer.php');
			$item = new stdClass();
			$item->editable = false;
			return SmartyRenderer::renderString($data, $params->layout->template->label);
		    break;
	    };		
	}

	static function getBody($data, $params)
	{
	    //$data is a row record
	    $item = new stdClass();
	    $item->editable = false;
	    
	    switch ($params->layout->mode) {
		case 'fields' :
		    $htmlRow = '<table class="table">';
		    $firstField = true;
		    foreach ($data as $key=>$value) {
			if ($firstField == true){
			    $item->label = $value;
			    $firstField = false;
			}
			$htmlRow .= '<tr><td>'.$key.'</td><td>'.$value.'</td></tr>' ;
		    }
		    $htmlRow .= '</table>';
		    return $htmlRow;
		    break;
		case 'template' :
		    require_once (NGEPATH_HELPERS . '/renderer/smarty/smartyrenderer.php');
			$item = new stdClass();
			$item->editable = false;
			return SmartyRenderer::renderString($data, $params->layout->template->body);
		    break;
	    };		
	}
	
	/**
	 * Get Available fields. Will be used in layout display for example
	 */
	static function getFields(){
	    
	    $fields = array();
	    $fields[] = array('name'=>'title', 'label'=>'Title', 'type'=>'text');
	    $fields[] = array('name'=>'author', 'label'=>'Author', 'type'=>'text');
	    $fields[] = array('name'=>'created', 'label'=>'Created date', 'type'=>'text');
	    $fields[] = array('name'=>'modified', 'label'=>'Modified date', 'type'=>'text');
	    $fields[] = array('name'=>'published', 'label'=>'Published date', 'type'=>'text');
	    $fields[] = array('name'=>'images.image_intro', 'label'=>'Intro image', 'type'=>'image');
	    $fields[] = array('name'=>'category', 'label'=>'Category', 'type'=>'text');
	    $fields[] = array('name'=>'introtext', 'label'=>'Intro text', 'type'=>'editor');
	    $fields[] = array('name'=>'fulltext', 'label'=>'Full text', 'type'=>'editor');
	    return $fields;
	}
	
	/**
	 * Loading single resultset
	 * @param type $value
	 * @param type $params
	 * @return type
	 */
	static function loadBody($value, $params=array()){
	    $query = $value;
	    try {
		$result = '';
		if ($query != ''){
			$db = JFactory::getDbo();
			$db->setQuery($query);
			$data = $db->loadObject();
			return self::getBody($data, $params);
		}
		else
			 return '';	    
	    }catch(Exception $e){
		trigger_error ( $e->getMessage(), E_USER_WARNING);
		return '';
	    }		
	}
	
	static function loadLinkList($params){
	    	    $query = $params->query;
	    try {
		$result = '';
		if ($query != ''){
			$db = JFactory::getDbo();
			$db->setQuery($query);
			//$result = $db->loadAssocList();
			$data = $db->loadRowList();
//			$result = $db->loadObjectList();
			
			$result = array();
			foreach($data as $dataRow){
			    $row = new stdClass();
			    for ($i=0; $i<count($dataRow);$i++){
				if ($i==0)
				    $row->id= $dataRow[$i];
				else	
				    if ($i==1)
				$row->title= $dataRow[$i];
				    else	
				    if ($i==2)
				$row->link= $dataRow[$i];
			    }
			    $result[] = $row;
			}
			return $result;
		}
		else
			 return null;	    
	    }catch(Exception $e){
		trigger_error ( $e->getMessage(), E_USER_WARNING);
		return null;
	    }

	}
	
	static function loadBodyList($params){
	    $query = $params->query;
	    try {
		$result = '';
		if ($query != ''){
			$db = JFactory::getDbo();
			$db->setQuery($query);
			//$result = $db->loadAssocList();
			$data = $db->loadRowList();
//			$result = $db->loadObjectList();
			
			$result = array();
			foreach($data as $dataRow){
			    $row = new stdClass();
			    for ($i=0; $i<count($dataRow);$i++){
				if ($i==0)
				    $row->title= $dataRow[$i];
				else	
				    if ($i==1)
				$row->body= $dataRow[$i];
			    }
			    $result[] = $row;
			}
			return $result;
		}
		else
			 return null;	    
	    }catch(Exception $e){
		trigger_error ( $e->getMessage(), E_USER_WARNING);
		return null;
	    }
	}
	
	static function loadThumbnailList($params){
	    $query = $params->query;
	    try {
		$result = '';
		if ($query != ''){
			$db = JFactory::getDbo();
			$db->setQuery($query);
			//$result = $db->loadAssocList();
			$data = $db->loadRowList();
//			$result = $db->loadObjectList();
			
			$result = array();
			foreach($data as $dataRow){
			    $row = new stdClass();
			    $row->header= new stdClass();
			    $row->header->label = '';
			    $row->header->glyph= new stdClass();
			    $row->header->glyph->name = '';
			    $row->badge= new stdClass();
			    $row->badge->label = '';
			    $row->hover='';
			    $row->legend='';	
			    for ($i=0; $i<count($dataRow);$i++){
				if ($i==0)
				    $row->header->label= $dataRow[$i];
				else	
				    if ($i==1)
				$row->body= $dataRow[$i];
			    }
			    $result[] = $row;
			}
			return $result;
		}
		else
			 return null;	    
	    }catch(Exception $e){
		trigger_error ( $e->getMessage(), E_USER_WARNING);
		return null;
	    }
	}
	
	/**
	 * when multiple content
	 * get a result with multiple items
	 * @param type $params
	 * @return type
	 */
	static function loadList($params)
	{		
	    $query = $params->query;
	    try {
		$result = '';
		if ($query != ''){
			$db = JFactory::getDbo();
			$db->setQuery($query);
			//$result = $db->loadAssocList();
			$data = $db->loadRowList();
//			$result = $db->loadObjectList();
			
			$result = array();
			foreach($data as $dataRow){
			    $row = new stdClass();
			    for ($i=0; $i<count($dataRow);$i++){
				if ($i==0)
				    $row->title= $dataRow[$i];
				else	
				    if ($i==1)
				$row->body= $dataRow[$i];
			    }
			    $result[] = $row;
			}
			return $result;
		}
		else
			 return null;	    
	    }catch(Exception $e){
		trigger_error ( $e->getMessage(), E_USER_WARNING);
		return null;
	    }
	}
	
}