    /**
     * @name saveModuleToArticle
     * @description save module in article like {loadposition xxx-nnn}
     * @param moduleId, articleId
     * @returns object or false
     */
    saveModuleToArticle = function (moduleId, articleId) {
    	var res = false;
    	
    	var url = "index.php?option=com_nge&task=module.moduletoarticle ";
    	var token = jQuery('#tokenform').find('input').attr('name');
    	var data = {};
    	data['m_id'] = moduleId;
    	data['a_id'] = articleId;
    	data[token] = 1;
    	
    	jQuery.ajax({
    	    url: url,
    	    data: data,
    	    async: false,
    	    success: function (resultJson) {
	    		try {
	    		    result = jQuery.parseJSON(resultJson);
	    		    res = result;
	    		} catch (err) {
	    		    bootbox.alert(COM_NGE_JS_EDITING_ARTICLE_UNABLE_MOVE_PART_TO_POSITION + err.toString());
	    		}
    	    },
    	    error: function (result) {
    	    	bootbox.alert(COM_NGE_JS_EDITING_ARTICLE_UNABLE_MOVE_PART_TO_MODULE);
    	    }
    	});
    	
    	return res;
    }
    
    /**
	 * @name savePartModuleToPartArticle
	 * @description Remove module div and saveArticle() with only div part content
	 * @param m_id
	 * @returns void
	 */
    savePartModuleToPartArticle = function(m_id) {
    	var partId = part.attr('id');
    	
    	/*Removing part and saving closest parent Layout (columns or page layout)    	*/
    	jQuery('#'+partId).remove();
    	
		var articleContent = jQuery('#'+partId).closest(".nge-article-content");
		saveArticle(articleContent);
    };
    
    /**
	 * @name removeArticlePart
	 * @description remove part in article and save the article
	 * @param part element
	 * @returns void
	 */
    removeArticlePart = function(part) {
		//var articleContent = part.closest(".nge-article-content");
		var $articleContent = jQuery(part).closest(".ngeinlineeditor");
    	/*Moving toolbar at the starting of the document, to avoid being removed with the part*/
    	jQuery('#part-toolbar').hide();
		
    	/*Removing part and saving closest parent Layout (columns or page layout)    	*/
    	part.remove();
    	
		saveInlineEditorContent($articleContent);
		//saveArticle(articleContent);
    };
    
    /**
	 * @name duplicateArticlePart
	 * @description duplicate part on the layout article and save the article
	 * @param pk, returnMode
	 * @returns void
	 */
    duplicateArticlePart = function(pk, returnMode) {
    	returnMode = returnMode || "preview";
    	var url="index.php?option=com_nge&task=part.duplicate";
    	url += '&pk='+pk;
    	url += '&return='+returnMode;	
    	url += '&media=layout';
    	
        jQuery.ajax({
    	    url:url,
    	    success:function(resultJson){
    	    	try{
		    result = jQuery.parseJSON(resultJson);
		    if (result.result == 1){
			var currentPart = jQuery("#part-" + pk);
			currentPart.after(result.html);

			var duplicatePart = jQuery('#part-' + result.pk);
			duplicatePart.addClass('ngeeditable');
			enableNgePartToolbar('#part-' + result.pk);
			enableNgeInlineEditing('#part-' + result.pk);
			enableNgeItemsAddable('#part-' + result.pk);
			enableNgeItemsSortable('#part-' + result.pk);

			activeSortableLayout(jQuery('#part-' + result.pk));

			var $articleContent = jQuery("#part-" + result.pk).closest(".ngeinlineeditor");
			saveInlineEditorContent($articleContent);
		    }
		    else {
			console.log(result.message);
		    }
    	    	} catch(err){
    	    		console.error("Error : Unable to duplicate part<br>"+err.toString());  
    	    	}
    	    },
    	    error:function(result){
    	    	console.error("Error : Unable to duplicate part");   
    	    }
    	});
    };

    /**
	 * @name afterPageLayoutPartCreate
	 * @description Called by config screen when new part has been created
	 * @returns void
	 */    
    afterArticleLayoutPartCreate = function(result){
         /*Finding current dropped part with the part-temp-dropped class*/
 		currentThumbTemporary = jQuery("#part-temp-dropped");
 		currentThumbTemporary.after(result.html);
 		currentThumbTemporary.remove();

		/*Add edit attributes and events for this new part*/
		enableNgePartToolbar('#part-'+result.pk);
		enableNgeInlineEditing('#part-'+result.pk);
		enableNgeItemsAddable('#part-'+result.pk);
		enableNgeItemsSortable('#part-'+result.pk);
		
		activeSortableLayout(jQuery('#part-'+result.pk));
		
		//var articleContent = jQuery("#part-"+result.pk).closest(".nge-article-content");
		//saveArticle(articleContent);
				
		var $inlineEditorContent = jQuery("#part-"+result.pk).closest(".ngeinlineeditor");
		saveInlineEditorContent($inlineEditorContent);
 	};

/*
	 * @name afterArticleLayoutModuleCreate
	 * @description Called by config screen when new module has been created
	 * @returns void
	 */    
    afterArticleLayoutModuleCreate = function(pk, content){

         /*Finding current dropped part with the part-temp-dropped class*/
 		currentThumbTemporary = jQuery("#part-temp-dropped");
 		currentThumbTemporary.after(content);
 		currentThumbTemporary.remove();
 	};

saveArticle = function(articleContent){
	var articleId = articleContent.closest(".ngeeditablestructurearticle").data('nge').id;
	var ckEditor = CKEDITOR.instances['ngelayout-article-content-'+articleId];
	var contentHtml = articleContent.clone(true);
	
	/*removing part preview and restore original part tags*/
	contentHtml.find('.part').each(function(){
	    var divid = jQuery(this).attr('id');
	    var pkArray = divid.split("-");
	    var pk 	= pkArray[1];
	    
	    var partlibrary = jQuery(this).attr('data-part');
	    jQuery(this).after('<part data-class="'+partlibrary+'" data-part="'+pk+'">&nbsp;</part>');
	    jQuery(this).remove();
	});

	/*Removing sortable, partToolbar stuff*/
	contentHtml.find('.nge-margin-handle').remove();
	contentHtml.find('.nge-part-margin-process').remove();
	contentHtml.find('.nge-part-resize-process').remove();
	contentHtml.find('.ui-draggable').removeClass('ui-draggable');
	contentHtml.find('.ui-resizable').removeClass('ui-resizable');
	contentHtml.find('.ui-sortable').removeClass('ui-sortable');
	contentHtml.find('.ngelayout-article-connected-sortable').removeClass('ngelayout-article-connected-sortable');
	
	var html = contentHtml.html();
	html = html.replace(/<!--{cke_protected}.*?-->/g, '');

	var url = 'index.php?option=com_nge&task=article.save';

	jQuery.ajax({
		url: url,
		data: {'a_id': articleId, 'articletext': html},
		type: 'POST',
		success: function (resultJson) {
			contentHtml.remove();
			try {
    		    result = jQuery.parseJSON(resultJson);

    		    if(result.success == 1){
        		    if(typeof ckEditor !== 'undefined')
        		    	nge.showNotification( result.message, 'success', 3000);
    		    }else{
    		    	if(typeof ckEditor !== 'undefined')
    		    		nge.showNotification( result.message, 'warning', 3000);
        		}
        		
    		} catch (err) {
    		    console.error("Error : to save article" + err.toString());
    		}
		},
		error: function (result) {
			console.error("Error : Article Wrong url, may be 500 Internal Servor Error =>" + result);
		}
	});       
};