/**
 * @name removePartPart
 * @description remove part and save layout part content
 * @param pk - id of the part
 * @returns void
 */
removePartPart = function (part) {
    var parentPartLayout = part.closest(".ngelayout-part");

    /*Moving toolbar at the starting of the document, to avoid being removed with the part*/
    jQuery('#part-toolbar').hide();
    
    /*Removing part and saving closest parent Layout (columns or page layout)    	*/
    part.remove();
    
    savePartContent(parentPartLayout);
};

/**
 * @name duplicatePartPart
 * @description duplicate part on the layout page content
 * @param pk - id of the part
 * @returns void
 */
duplicatePartPart = function (pk, returnmode) {
    returnmode = returnmode || "preview";
    var url = "index.php?option=com_nge&task=part.duplicate";
    url += '&pk=' + pk;
    url += '&return=' + returnmode;
    url += '&media=layout';

    jQuery.ajax({
        url: url,
        success: function (resultJson) {
            try {
                result = jQuery.parseJSON(resultJson);

                var currentPart = jQuery("#part-" + pk);
                currentPart.after(result.html);

                /*Finding closest parent Layout (columns or page layout)*/
                var parentPartLayout = jQuery('#part-' + result.pk).closest(".ngelayout-part");

    	    	enableNgePartToolbar('#part-'+result.pk);
    		enableNgeInlineEditing('#part-'+result.pk);
		enableNgeItemsAddable('#part-'+result.pk);
		enableNgeItemsSortable('#part-'+result.pk);
			
    		activeSortableLayout(jQuery('#part-'+result.pk));

		savePartContent(parentPartLayout);
            } catch (err) {
                bootbox.alert(COM_NGE_JS_EDITING_UNABLE_DUPLICATE_PART + "<br>" + err.toString());
            }
        },
        error: function (result) {
            bootbox.alert(COM_NGE_JS_EDITING_UNABLE_DUPLICATE_PART);
        }
    });
};

/**
 * @name afterPartLayoutPartCreate
 * @description Called by config screen when new part has been created
 * @returns void
 */
afterPartLayoutPartCreate = function (result) {
    /*Finding current dropped part with the part-temp-dropped class*/
    currentThumbTemporary = jQuery("#part-temp-dropped");
    currentThumbTemporary.after(result.html);
    currentThumbTemporary.remove();

	/*Add dit attributes and events for this new part*/
	enableNgePartToolbar('#part-'+result.pk);
	enableNgeInlineEditing('#part-'+result.pk);
	enableNgeItemsAddable('#part-'+result.pk);
	enableNgeItemsSortable('#part-'+result.pk);
		
	enablePartSortableLayout('#part-'+result.pk);
	
    var parentPartLayout = jQuery('#part-' + result.pk).closest(".ngelayout-part");
    savePartContent(parentPartLayout);
};
    
/**
 * @name savePartContent
 * @description saving layout part content
 * @returns void
 */
savePartContent = function (parentPartLayout) {
    var value = cleanEditingInEditorContent(jQuery(parentPartLayout).clone(true));

    /*Part ID : finding closest parent part. part id is "part-nnn"*/
    var divid = jQuery(parentPartLayout).closest(".part").attr('id');
    var pkArray = divid.split("-");
    var pk = pkArray[1];

    /*Part property*/
    var property = jQuery(parentPartLayout).data('nge').source;
    property = property.split(":")[1];

    storePartProperty(pk, 'content', property, value);

    disableSortableDragDropZone();
};